var fs = require('fs');
var path = require('path');
var util = require('util');

var regexps = require('../');

var splitter = '<!-- CUT -->\n';
var readmePath = path.resolve(__dirname, '../README.md');

var content = fs.readFileSync(readmePath, {
	encoding: 'UTF-8'
});

var docHeader = content.split(splitter).shift() + splitter;

var docPath = path.resolve(__dirname, 'doc/');

var output = fs.readdirSync(docPath).map(function(filename) {
	var propname = path.basename(filename, '.json');
	var data = JSON.parse(
		fs.readFileSync(path.resolve(docPath, filename), {
			encoding: 'UTF-8'
		})
	);

	var output = [
		'## ' + propname,
		'\n' + data.description
	].join('\n');

	if (undefined !== data.examples) {
		var examplesOutput = [
			'\n\n### Examples',
			'\n```js',
			'var regexps = require(\'twitter-regexps\');\n'
		];

		if (undefined !== data.examples.exec) {
			examplesOutput.push(
				data.examples.exec.map(function(test) {
					var result = regexps[propname].exec(test.input);
					var output = [
						'var data = regexps.' + propname + '.exec(\'' + test.input + '\');'
					];

					var maxCommentLength = Math.max.apply(
						null,
						Object.keys(test.args).map(function(key) { return test.args[key].length })
					);

					var maxKeyLength = Math.max.apply(
						null,
						Object.keys(test.args).map(function(key) { return key.length })
					);

					Array.prototype.push.apply(
						output,
						Object.keys(test.args).map(function(key) {
							var comment = test.args[key];
							var commentLength = comment.length;
							var keyPadding = maxKeyLength - key.length;

							if (commentLength < maxCommentLength) {
								comment += Array(maxCommentLength - commentLength + 1).join(' ');
							}

							return [
								'/* ' + comment + ' */ data.' + key + ';' + Array(keyPadding + 1).join(' '),
								util.inspect(result[key])
							].join(' // ');
						})
					);

					return output.join('\n');
				})
			);
		}

		examplesOutput.push('```');

		output += examplesOutput.join('\n');
	}

	return output;
}).join('\n\n');

fs.writeFileSync(readmePath, [docHeader, output].join('\n'));
